% % This code simulates a market for entrepreneurs and workers with double
% % adverse selection and learning. Its base version (i.e. without learning)
% % is meant to mimic the outcomes predicted in Boadway and Sato (2011).
% %
% % The code was tested on Matlab (R2015a) and GNU Octave (4.0.3).
% % To improve reproducibility, this version complies with Octave.
% %
% % Author: Diego d'Andria
% % This version: February 2017



%% GENERATE POPULATIONS AND DECLARE MATRIXES TO STORE VALUES

% True abilities/qualities and estimated ones are stored in these matrices:
abilities = zeros(1,pop_entrepreneurs);
qualities = zeros(1,pop_workers);
entrepreneurs_estimated_a = zeros(periods, pop_entrepreneurs);
workers_estimated_q = zeros(periods, pop_workers);

% Ability and quality are uniformly distributed in [0,1] and rounded to the
% nearest second decimal. Therefore there are 100 levels of ability or quality.
% In period 1 all agents are given their population average values.
for i=1:pop_entrepreneurs
    abilities(1,i) = round((rand*100))/100;
    if abilities(1,i) == 0;
      abilities(1,i) = 0.01;
    end
end
if learning_type >= 0
  for i=1:pop_entrepreneurs
    entrepreneurs_estimated_a(1,i) = mean(abilities);
  end
elseif learning_type == -1
  for i=1:pop_entrepreneurs
    entrepreneurs_estimated_a(1,i) = abilities(1,i);
  end
end  

for j=1:pop_workers
  qualities(1,j) = round((rand*100))/100;
    if qualities(1,j) == 0;
      qualities(1,j) = 0.01;
    end
end
if learning_type >= 0 
  for j=1:pop_workers
    workers_estimated_q(1,j) = mean(qualities);
  end
elseif learning_type == -1 
  for j=1:pop_workers
    workers_estimated_q(1,j) = qualities(1,j);
  end
end

% These matrices take value 1 if the agent decides for entry:
entrepreneurs_entry = zeros(periods, pop_entrepreneurs);
workers_entry = zeros(periods, pop_workers);

% This matrices keeps track of workers' employers in each period:
workers_employed = zeros(periods, pop_workers);
workers_employed_binary = zeros(periods, pop_workers);
hired_workers_no = zeros(periods, pop_entrepreneurs);

% These vectors store, for each period, the expected productivity and wages
% that entrepreneurs use to decide about their entry:
temp_quality = zeros(1, pop_workers);
avg_quality = zeros(1,periods);
avg_wage = zeros(1,periods);

% Average quality and wage for each entrepreneur:
entrepreneurs_sumq = zeros(periods, pop_entrepreneurs);
entrepreneurs_sumw = zeros(periods, pop_entrepreneurs);

% This matrix takes value 1 if the entrepreneurial project succeeded:
entrepreneurs_success = zeros(periods, pop_entrepreneurs);
% If the entrepreneurial project succeeded, this matrix stores the net profit:
entrepreneurs_profit = zeros(periods, pop_entrepreneurs);

% This matrix stores the interest rates (including risk premia) demanded
% to entrepreneurs, which are function of their assessed ability:
interest_rate = zeros(periods, pop_entrepreneurs);

% This matrix stores the quality signal for workers observed in each period: 
workers_observed_q = zeros(periods, pop_workers);

% These matrices are used to compute welfare outcomes at the end.
welfare_expost = zeros(1,periods);
welfare_expost_entr = zeros(1,periods);
welfare_expost_work = zeros(1,periods);
welfare_expost_gov = zeros(1,periods);
welfare_entered_entr = zeros(1,periods);
welfare_costs_entr = zeros(1,periods);


% ------------------------------------------------------------------------
%% MAIN LOOP
% ------------------------------------------------------------------------

tic;

for t=1:periods
    
    % On-screen message to keep track of the code running:
    printf("Now simulating period: ");
    disp(t);
    
    %% CREDIT MARKET
    % Determine the lending rates to entrepreneurs
    for i=1:pop_entrepreneurs
        if entrepreneurs_estimated_a(t,i)>0
            interest_rate(t,i) = rho/entrepreneurs_estimated_a(t,i);
        else
            % set a cap to the interest rate to avoid infinite values:
            interest_rate(t,i) = interest_cap;
        end
        if interest_rate(t,i)>interest_cap
            interest_rate(t,i) = interest_cap;
        end
    end
    
    %% ENTRY
    % Agents compare expected payoff with their alternative option. The
    % expected payoff is function of the ability or quality they are
    % assigned by other agents' estimation.
    % Entrepreneurs compute expected profit based on the average quality of
    % workers who entered the sector.
	% Without information, workers expect to receive the average wage in sector E.
	% With learning, they expect to earn the wage corresponding to their assessed quality.
	% With full information, wage in sector E is equal to individual quality.
    
    % workers entry
	if learning_type ~= 0 || (learning_type == 0 && t == 1)
    for j=1:pop_workers
        if workers_estimated_q(t,j)*(1+(pit_tax*(1-pit_tax_incidence))) >= qualities(1,j)
            workers_entry(t,j) = 1;
        end
        temp_quality(1,j) = workers_estimated_q(t,j)*workers_entry(t,j);
    end
	elseif learning_type == 0
	    for j=1:pop_workers
        if avg_wage(1,t)*(1+(pit_tax*(1-pit_tax_incidence))) >= qualities(1,j)
            workers_entry(t,j) = 1;
        end
        temp_quality(1,j) = workers_estimated_q(t,j)*workers_entry(t,j);
    end
	end
    
    % entrepreneurs entry
    % "avg_quality" calculates the average assessed quality of workers who
    % opted for the entrepreneurial sector.
    n_sum = sum(temp_quality~=0);
    n_sum(n_sum==0) = NaN;
    avg_quality(1,t) = sum(temp_quality) ./ n_sum;
    for i=1:pop_entrepreneurs
        if abilities(1,i)*(R*avg_quality(1,t)^alpha - interest_rate(t,i) ...
                *n*(1-pit_tax*pit_tax_incidence)*avg_quality(1,t)) +cit_tax >= alt_profit
            entrepreneurs_entry(t,i) = 1;
        end
    end

	% Set the average wage earned by workers in sector E as a basis for entry 
	% decisions in period t+1: 
	if learning_type == 0 && t < periods
		avg_wage(1,t+1) = avg_quality(1,t);
	end
    
    
    %% MATCHING
    % Matching of entrepreneurs and workers, based on a sorting algorithm.
    % The best-looking entrepreneur hires the n best-looking workers, then
    % the second-best hires the n second-best, and so on.
    ranked_entrepreneurs(:,1)=1:pop_entrepreneurs;
    ranked_entrepreneurs(:,2)=entrepreneurs_entry(t,:);
    ranked_entrepreneurs(:,3)=entrepreneurs_estimated_a(t,:);
    ranked_entrepreneurs = sortrows(ranked_entrepreneurs,[-2,-3]);
    
    ranked_workers(:,1)=1:pop_workers;
    ranked_workers(:,2)=workers_entry(t,:);
    ranked_workers(:,3)=workers_estimated_q(t,:);
    ranked_workers = sortrows(ranked_workers,[-2,-3]);
    
    for j=1:pop_workers
        if workers_entry(t,j)==1
            [row,col,v] = find(ranked_workers(:,1)==j);
            workers_employed(t,j) = ranked_entrepreneurs(ceil(row/n),1);
            if workers_employed(t,j)>0
              workers_employed_binary(t,j) = 1;
            end
            if entrepreneurs_entry(t,workers_employed(t,j))== 0
                workers_employed(t,j) = 0;
                workers_employed_binary(t,j) = 0;
            end
        end
    end
    
    
    %% PRODUCTION
    % Here it is determined whether enterprises succeed or not. Wages are
    % paid upfront to workers.
    for i=1:pop_entrepreneurs
        hired_workers_q = zeros(1,pop_workers);
        hired_workers_estq = zeros(1,pop_workers);
        if rand<=abilities(1,i) && entrepreneurs_entry(t,i) == 1
            entrepreneurs_success(t, i) = 1;
            for j=1:pop_workers
                if workers_employed(t,j)==i
                    hired_workers_no(t,i) = hired_workers_no(t,i) + 1;
                    hired_workers_q(1,j) = qualities(1,j);
                    if learning_type ~=0
						hired_workers_estq(1,j) = workers_estimated_q(t,j);
					elseif learning_type ==0
						hired_workers_estq(1,j) = avg_wage(1,t);
					end
                else
                    hired_workers_q(1,j) = 0;
                    hired_workers_estq(1,j) = 0;
                end
            end
            entrepreneurs_sumq(t,i) = sum(hired_workers_q);
            entrepreneurs_sumw(t,i) = sum(hired_workers_estq);
            
            % Gross profits are obtained from true qualities, while wages depend on estimated qualities:
            if hired_workers_no(t,i)>0
                entrepreneurs_profit(t,i) = entrepreneurs_success(t, i)*(R*entrepreneurs_sumq(t,i)^alpha ...
                    - (1-pit_tax*pit_tax_incidence)*interest_rate(t,i)*entrepreneurs_sumw(t,i)) + cit_tax;
            else
                entrepreneurs_profit(t,i) = 0;
            end
        end
        
    end
    
    
    
    %% LEARNING
    % Entrepreneurs' abilities are assessed based on the share of successes
    % over total projects started.
    % Workers' qualities can be assessed either by means of a noisy signal.
    if learning_type > 0 
        
        if t>=learning_starts_at && t<periods
            
            %% Abilities
            sum_success = sum(entrepreneurs_success,1);
            sum_entry = sum(entrepreneurs_entry,1);
            for i=1:pop_entrepreneurs
                if sum_entry(i)>0
                    entrepreneurs_estimated_a(t+1, i)= ...
                        round((((sum_success(i)/sum_entry(i))+normrnd(0,learning_anoise))*100))/100;
                    if entrepreneurs_estimated_a(t+1, i)>=1
                        entrepreneurs_estimated_a(t+1, i)=0.99;
                    elseif entrepreneurs_estimated_a(t+1, i)<=0
                        entrepreneurs_estimated_a(t+1, i)=0.01;
                    end
                elseif t>1
                    entrepreneurs_estimated_a(t+1, i)=entrepreneurs_estimated_a(t, i);
                end
            end
            
            %% Qualities
            if learning_type>0
                for j=1:pop_workers
                    
                    % Agents observe an imperfect signal of a worker quality:                    
                    if learning_type==1
                      workers_observed_q(t,j)=qualities(1,j)+normrnd(0,learning_qnoise);
                    end
                     
                    if workers_observed_q(t,j)>=1
                        workers_observed_q(t,j)=0.99;
                    elseif workers_observed_q(t,j)<=0
                        workers_observed_q(t,j)=0.01;
                    end
                    
                    % Here the Bayesian inference occurs. The code calls a
                    % function named "Bayesian_learning" stored in a
                    % separate file, which returns the estimated probability
					% that a given quality value is the right one, given signals
					% observed in the past. the estimated quality corresponds to
					% the largest of these probabilities:
                    observations = workers_observed_q(1:t,j);
                    observations = observations(observations ~= 0);
                    if sum(observations) > 0
                        results = Bayesian_learning( observations, learning_qnoise );
                        [maxValue, I] = max(results(:));
                        [rowsOfMaxes, colsOfMaxes] = find(results == maxValue);
                        if numel(rowsOfMaxes)==1
                            workers_estimated_q(t+1,j)=rowsOfMaxes/100;
                        elseif numel(rowsOfMaxes)>1
                            % If some probabilities are equal, use the mean quality:
							workers_estimated_q(t+1,j)= round(mean(rowsOfMaxes))/100;
                        else
                            h = errordlg('An error occurred in the Bayesian learning algorithm!');
                            % disp('An error occurred in the Bayesian learning algorithm!')
                            return
                        end
                    else
                        workers_estimated_q(t+1,j)=workers_estimated_q(t,j);
                    end
                end
               
            end
            
            
        elseif t<learning_starts_at % Learning only occurs after some periods
            for i=1:pop_entrepreneurs
                entrepreneurs_estimated_a(t+1,i)=entrepreneurs_estimated_a(t,i);
            end
            for j=1:pop_workers
                workers_estimated_q(t+1,j)=workers_estimated_q(t,j);
            end
            
        end
        
    elseif learning_type==0 | learning_type==-1
        for i=1:pop_entrepreneurs
            entrepreneurs_estimated_a(t+1,i)=entrepreneurs_estimated_a(t,i);
        end
        for j=1:pop_workers
            workers_estimated_q(t+1,j)=workers_estimated_q(t,j);
        end
        
    end
    
    
    %% PERFORMANCE INDICATORS
    
    % Compute welfare (as surplus) for the period
    for i=1:pop_entrepreneurs
        welfare_expost_entr(1,t) = welfare_expost_entr(1,t) + entrepreneurs_profit(t,i) -alt_profit;
        welfare_entered_entr(1,t) = welfare_entered_entr(1,t)+entrepreneurs_entry(t,i);
        welfare_costs_entr(1,t) = welfare_costs_entr(1,t) + interest_rate(t,i)*entrepreneurs_sumw(t,i);
    end
    
    for j=1:pop_workers
        if workers_employed(t,j)>0
            welfare_expost_work(1,t) = welfare_expost_work(1,t) + workers_estimated_q(t,j) -qualities(1,j);
        end
    end
    
    welfare_expost_gov(1,t) = -cit_tax*welfare_entered_entr(1,t) - pit_tax*welfare_costs_entr(1,t);
    
    welfare_expost(1,t) = welfare_expost_entr(1,t) + rho*welfare_expost_work(1,t);
    
    
    
end  % end of the main loop.

time_run = toc;
fprintf("Total time used for running the main loop was %.0f seconds \n", time_run ); 

% ------------------------------------------------------------------------


% Generate here other welfare measures 

welfare_sum_allperiods = 0;
welfare_npv_allperiods = 0;
welfare_sum_allp_entr = 0;
welfare_sum_allp_work = 0;
welfare_sum_allp_gov = 0;
entry_sum_entr = zeros(1,periods);
employed_sum_work = zeros(1,periods);


  
for t=1:periods
  welfare_sum_allperiods = welfare_sum_allperiods + welfare_expost(1,t);
  welfare_npv_allperiods = welfare_sum_allperiods + welfare_expost(1,t)/rho^t;
  welfare_sum_allp_entr = welfare_sum_allp_entr + welfare_expost_entr(1,t);
  welfare_sum_allp_work = welfare_sum_allp_work + welfare_expost_work(1,t);
  welfare_sum_allp_gov = welfare_sum_allp_gov + welfare_expost_gov(1,t);
  entry_sum_entr(1,t) = sum(entrepreneurs_entry(t,:));
  employed_sum_work(1,t) = sum(workers_employed_binary(t,:));
end


RESULTS_welfare_tot(rep,1) = welfare_sum_allperiods;
RESULTS_welfare_entr(rep,1) = welfare_sum_allp_entr;
RESULTS_welfare_work(rep,1) = welfare_sum_allp_work;
RESULTS_welfare_gov(rep,1) = welfare_sum_allp_gov;
RESULTS_entry_entr(rep,1) = sum(entry_sum_entr)/(pop_entrepreneurs*periods);
RESULTS_employed_work(rep,1) = sum(employed_sum_work)/(pop_workers*periods);


% Clean the workspace from items that are not used any more:
clear I avg_quality avg_wage col colsOfMaxes hired_workers_estq hired_workers_q i
clear j maxValue n_sum observations ranked_entrepreneurs ranked_workers results
clear row rowsOfMaxes sum_entry sum_success t temp_quality v entry_sum_entr employed_sum_work
clear welfare_sum_allp_entr welfare_sum_allp_work welfare_sum_allp_gov


%% END OF FILE
%%